/*
 * Copyright (C) 2013 Couchbase, Inc.
 * All rights reserved.
 */
package com.couchbase.demo;


import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * SleepingTester is designed to sleep for a random period of time and then
 * issue various operations to the server, much like a real user's code would
 * when running in an app server. During the sleepy time, a real application
 * would be sending HTML to a user, calling other databases and web services, or
 * filling the log file.
 *
 * There are many places here where we should have better error handling.
 *
 * @author Matt Ingenthron <matt@couchbase.com>
 */
public class Noisemaker implements Runnable {

  final int desiredThroughput;
  private int getSuccess, getFailure, setSuccess, setFailure;
  private long getElapsedTime, runMethodElapsed;
  private String noisyString;
  static Random random = new Random();
  volatile boolean finished = false;
  private int counter = 0;

  public Noisemaker(int desiredThroughput) {
    this.desiredThroughput = desiredThroughput;
    noisyString = random.makeAString(512, 1024);

  }

  public void stopMe() {
    finished = true;
  }

  @Override
  public void run() {
    String threadName = Thread.currentThread().getName();

    Logger.getLogger(Noisemaker.class.getName()).log(Level.INFO,
            "Thread {0} starting.", threadName);


    while (true) {
      try {
        if (finished) {
          Logger.getLogger(Noisemaker.class.getName()).log(Level.INFO,
                  "Thread {0} stopping.", threadName);
          break;
        }

        if (random.random(0, 1) == 0) {

          String workingKey = threadName + ":" + counter++;

          try {
            CouchbaseCheck.getTheClient().set(workingKey, 600, noisyString).get();
            setSuccess++;
          } catch (ExecutionException ex) {
            setFailure++;
            Logger.getLogger(Noisemaker.class.getName()).log(Level.SEVERE, "ExecutionExeception during set.", ex);
          }

        } else {
          String workingKey = threadName + ":" + random.random(1,counter);

          try {
            CouchbaseCheck.getTheClient().asyncGet(workingKey).get(2500, TimeUnit.MILLISECONDS);
            getSuccess++;
          } catch (ExecutionException ex) {
            getFailure++;
            Logger.getLogger(Noisemaker.class.getName()).log(Level.SEVERE, "ExecutionExeception during set.", ex);
          } catch (TimeoutException ex) {
            getFailure++;
            Logger.getLogger(Noisemaker.class.getName()).log(Level.SEVERE, "TimeoutExeception during set.", ex);
          }

        }


        // sleep just enough to try to the target throughput
        int sleepVal = 1000 / desiredThroughput;
        sleepVal = random.random(sleepVal / 2, sleepVal + sleepVal / 2);

        Thread.sleep(sleepVal);

      } catch (InterruptedException ex) {
        Logger.getLogger(Noisemaker.class.getName()).log(Level.SEVERE, "Sleeping interrupted.", ex);
      }

    }

    String wrongGetLatency = String.format("%1$,.2f", (getElapsedTime / 1000000.0) / (getSuccess + getFailure));
    String wrongSetLatency = String.format("%1$,.2f", (runMethodElapsed / 1000000.0) / (setSuccess + setFailure));

    Logger.getLogger(Noisemaker.class.getName()).log(Level.INFO,
            "Thread {0} completed all of it's sets and gets. "
            + "Get successes: " + getSuccess + "; get failures: " + getFailure
            + " Set successes: " + setSuccess + "; set failures: " + setFailure
            + " Get rough latency average: " + wrongGetLatency + "ms"
            + " Set rough latency average: " + wrongSetLatency + "ms",
            threadName);
  }
}
