/*
 * Copyright (C) 2011 Couchbase, Inc.
 * All rights reserved.
 */
package com.couchbase.demo;

import com.couchbase.client.CouchbaseClient;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * SleepingTester is designed to sleep for a random period of time and then
 * issue various operations to the server, much like a real user's code would
 * when running in an app server. During the sleepy time, a real application
 * would be sending HTML to a user, calling other databases and web services, or
 * filling the log file.
 *
 * There are many places here where we should have better error handling.
 *
 * @author Matt Ingenthron <matt@couchbase.com>
 */
public class SleepingTester implements Runnable {

  final int times;
  private int getSuccess, getFailure, setSuccess, setFailure;
  private long getElapsedTime, setElapsedTime;
  private String testString;
  static Random random = new Random();
  volatile boolean finished = false;
  private long counter = 0L;
  CouchbaseClient cbclient;

  public SleepingTester(int iterations) {
    times = iterations;
    testString = random.makeAString(512, 1024);
    cbclient = CouchbaseCheck.getTheClient();

  }

  public void stopMe() {
    finished = true;
  }


  @Override
  public void run() {
    String threadName = Thread.currentThread().getName();

    Logger.getLogger(SleepingTester.class.getName()).log(Level.INFO,
            "Thread {0} starting.", threadName);
    for (int i = 0; i < times; i++) {
      if (finished) {
        Logger.getLogger(SleepingTester.class.getName()).log(Level.INFO,
                "Thread {0} stopping.", threadName);
        break;
      }
      try {
        String workKey = threadName + ":" + counter++;
        cbclient.asyncGet(workKey).get(2500, TimeUnit.MILLISECONDS);
        getSuccess++;
        if (counter %1000 == 0 ) {
          System.out.println("Completed " + counter + " gets with " + getSuccess + " successes and " + getFailure + " failures.");
        }
      } catch (InterruptedException ex) {
        Logger.getLogger(SleepingTester.class.getName()).log(Level.WARNING,
                "Thread waiting to do more terminated.", ex);
      } catch (ExecutionException ex) {
        Logger.getLogger(SleepingTester.class.getName()).log(Level.SEVERE,
                "Thread trying to work hit a problem, trying to continue.", ex);
      } catch (TimeoutException e) {
//        Logger.getLogger(SleepingTester.class.getName()).log(Level.WARNING,
//                "Unexpected Exception in the main run() for loop", e);
        getFailure++;
      }
    }

    String wrongGetLatency = String.format("%1$,.2f", (getElapsedTime / 1000000.0) / (getSuccess + getFailure));
    String wrongSetLatency = (setSuccess > 0 || setFailure > 0) ? String.format("%1$,.2f", (setElapsedTime / 1000000.0) / (setSuccess + setFailure)) : "0";

    Logger.getLogger(SleepingTester.class.getName()).log(Level.INFO,
            "Thread {0} completed all of it's sets and gets. "
            + "Get successes: " + getSuccess + "; get failures: " + getFailure
            + " Set successes: " + setSuccess + "; set failures: " + setFailure
            + " Get rough latency average: " + wrongGetLatency + "ms"
            + " Set rough latency average: " + wrongSetLatency + "ms",
            threadName);
  }



}
